#include <SoftwareSerial.h>
#include <TinyGPS.h>
#include <LiquidCrystal_I2C.h>
#include <SPI.h>
#include <SD.h>

LiquidCrystal_I2C lcd (0x27,16,2); // I2C address should be correct!!

/*
   SD card attached to SPI bus as follows:
 ** MOSI - pin 11
 ** MISO - pin 12
 ** CLK - pin 13
 ** CS - pin 4 (for MKRZero SD: SDCARD_SS_PIN)
*/

const int chipSelect = 4;

TinyGPS gps;
SoftwareSerial ss(6, 5);

File dataFile; 

void setup() {
  Serial.begin(9600);
  ss.begin(9600);

  lcd.init();
  lcd.backlight();

  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB port only
  }
  
  Serial.print("Initializing SD card...");

  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    while (1);
  }
  Serial.println("card initialized."); 
}

void loop() {
  bool newData = false;
  String lat_msg, lon_msg, stmp;

  int years;
  byte months, days, hours, minutes, seconds, hundredths;

  dataFile = SD.open("Loc.txt", FILE_WRITE);
  if (!dataFile) {
    lcd.setCursor(0, 0);
    lcd.print("File error");
    while (1){
      delay(1000); 
    }
  } 
  
  // For one second we parse GPS data and report some key values
  for (unsigned long start = millis(); millis() - start < 1000;) {
    while (ss.available()) {
      char c = ss.read();
      // Serial.write(c); // uncomment this line if you want to see the GPS data flowing
      if (gps.encode(c)) // Did a new valid sentence come in?
        newData = true;
    }
  }

  lcd.init();
  
  if (newData) {
    float flat, flon;
    
    gps.f_get_position(&flat, &flon);

    lat_msg = "";
    lat_msg += String(flat, 5);
    lcd.setCursor(0, 0);
    lcd.print(lat_msg);   
    lat_msg += ",";
     
    lon_msg = "";
    lon_msg += String(flon, 5);
    lcd.setCursor(0, 1);
    lcd.print(lon_msg);
    lon_msg += ",";

    gps.crack_datetime(&years,&months,&days,&hours,&minutes,&seconds,&hundredths);

    stmp = "";
    stmp += String(years);
    stmp += "/";
    stmp += String(months);
    stmp += "/";
    stmp += String(days);
    stmp += "-";
    stmp += String(hours);
    stmp += ":";
    stmp += String(minutes);
    stmp += ":";
    stmp += String(seconds);

    dataFile.println(lat_msg+lon_msg+stmp); 
  }
  else {
    lcd.setCursor(0, 0);
    lcd.print("No signal");
  }
  dataFile.close(); 
}
